#include "CGame.h"

#include "GameIncludes.h"

CGame::CGame(void) :
_gameStarted(false),
_score(0)
{
	_mainMenuLevel = CSerializer::getInstance()->loadLevelFromXmlFile("media/mainmenu.xml");
	_endMenuLevel = CSerializer::getInstance()->loadLevelFromXmlFile("media/endmenu.xml");
	_currentLevel = CSerializer::getInstance()->loadLevelFromXmlFile("media/level1.xml");
	_hudLevel = CSerializer::getInstance()->loadLevelFromXmlFile("media/hud.xml");
	_pauseMenuLevel = CSerializer::getInstance()->loadLevelFromXmlFile("media/pausemenu.xml");

	_mainMenuLevel->_bShow = true;
	_hudLevel->_bShow = true;
}


CGame::~CGame(void)
{
}

CGame* CGame::getInstance()
{
	static CGame instance;
	return &instance;
}

void CGame::run()
{
	_screenSize = Vector2i(800, 600);
	RenderWindow window(VideoMode(_screenSize.x, _screenSize.y), "Astenrods - LD27 - by GuillaumeGwi");

	_frameClock.restart();
	_gameTime = 0.0f;
	float lastCopyTime = _gameTime;
	bool bShowDebug = false;
	float simulationTime = 0.0f;
	_reverseCoeff = 1.0f;
	// on fait tourner le programme jusqu' ce que la fentre soit ferme
	while (window.isOpen())
	{
		// on inspecte tous les vnements de la fentre qui ont t mis depuis la prcdente itration
		Event event;
		while (window.pollEvent(event))
		{
			// vnement "fermeture demande" : on ferme la fentre
			switch (event.type)
			{
			case Event::Closed:
				{
					window.close();
					break;
				}
			case Event::KeyPressed:
				{
					if (event.key.code == Keyboard::Escape)
					{
						if (_mainMenuLevel->_bShow)
						{
							window.close();
						}
						else
						{
							this->stopGame();
						}
					}
					else if (event.key.code == Keyboard::D)
					{
						bShowDebug = !bShowDebug;
					}
					else if (event.key.code == Keyboard::Up)
					{
						_reverseCoeff *= 1.5f;
					}
					else if (event.key.code == Keyboard::Down)
					{
						_reverseCoeff /= 1.5f;
					}
					else if (event.key.code == Keyboard::RShift || event.key.code == Keyboard::LShift)
					{
						_reverseCoeff = 1.0f;
						_endMenuLevel->_bShow = false;
					}
					else if (event.key.code == Keyboard::P)
					{
						this->togglePause();
					}
					else if (event.key.code == Keyboard::Return && _mainMenuLevel->_bShow || event.key.code == Keyboard::R)
					{
						this->startGame();
					}
					break;
				}
			}
		}

		window.clear();

		//update game
		float DeltaTime = _frameClock.getElapsedTime().asSeconds();
		_frameClock.restart();
		if (DeltaTime >= 1.0f / 30.0f)
		{
			DeltaTime = 1.0f / 30.0f;
		}

		CProfiler::getInstance()->registerProfilerTime("DeltaTime", DeltaTime);
		simulationTime += DeltaTime;
		bool bUpdate = false;
		if (simulationTime >= 1.0f / 60.0f)
		{
			bUpdate = true;
			CProfiler::getInstance()->registerProfilerTime("Simulation Time", simulationTime);
		}

		bool reverseMode = sf::Keyboard::isKeyPressed(Keyboard::RShift) || sf::Keyboard::isKeyPressed(Keyboard::LShift);
		_reverseCoeff = CMath::Clamp(_reverseCoeff, 0.25f, 2.0f);

		if (_gameStarted)
		{
			_bPause = _endMenuLevel->_bShow || _pauseMenuLevel->_bShow;

			if (bUpdate)
			{
				if (reverseMode)
					simulationTime *= -1.0f * _reverseCoeff;

				if (!_bPause)
				{	
					int oldGameTimeRounded = int(_gameTime);
					_gameTime += simulationTime;
					int newGameTimeRounded = int(_gameTime);
					this->addScore((newGameTimeRounded - oldGameTimeRounded) * 10);
				}

				CProfiler::getInstance()->registerProfilerTime("Game Time", _gameTime);

				if (_gameTime <= 0.0f)
				{
					_gameTime = 0.0f;
					simulationTime = 0.0f;
				}

				if (!_bPause)
				{
					CScopeProfiler profiler("Recorder Update");
					CRecorder::getInstance()->update(simulationTime);
				}

				if (!_bPause && _currentLevel && _currentLevel->_rootGroup)
				{
					float copyInterval = getActualDifficultyParams()._copyInterval;
					if (!reverseMode && _gameTime - lastCopyTime >= copyInterval)
					{
						lastCopyTime = _gameTime;
						//create a copy of main node
						for (auto ItObject = CRecorder::getInstance()->_recordingObjects.begin(); ItObject != CRecorder::getInstance()->_recordingObjects.end(); ++ItObject)
						{
							if ((*ItObject)->findFirstNodeBehaviorsOfClass<CPlayerControllerBH>())
							{
								//Create copy
								CNode* copyNode = CSerializer::getInstance()->loadFromXmlFile("media/templates/testcopy.xml");
								if (copyNode)
								{
									//remove BHs
									copyNode->clearNodeBehaviors();
									//add to replaying object
									bool bSpawned = false;
									float TestTime = _gameTime - copyInterval;
									while (!bSpawned && TestTime >= 0.0f)
									{
										if (!CRecorder::getInstance()->isObjectBasedOnPlayedAround(*ItObject, TestTime, 1.0f))
										{
											CRecorder::getInstance()->addReplayingObject(copyNode, *ItObject, TestTime);
											bSpawned = true;

											//add to the level
											this->spawnNode(copyNode);
										}
										else
										{
											TestTime -= 2.0f;
										}
									}
									
									if (!bSpawned)
									{
										this->definitelyDestroyNode(copyNode);
									}
								}
							}
						}
					}
					else if (reverseMode)
					{
						if (_gameTime < lastCopyTime)
						{
							lastCopyTime = _gameTime - copyInterval;
							if (lastCopyTime < 0.0f)
								lastCopyTime = 0.0f;
						}

						// respawn deleted nodes
						{
							auto ItLB = _deletedNodes.lower_bound(_gameTime + simulationTime);
							for (auto ItObject = ItLB; ItObject != _deletedNodes.end(); )
							{
								this->addScore(-ItObject->second->getScore());
								this->addToScene(ItObject->second);
								ItObject = _deletedNodes.erase(ItObject);
							}
						}

						//Unspawn spawned nodes
						{
							auto ItLB = _spawnedNodes.lower_bound(_gameTime + simulationTime);
							for (auto ItObject = ItLB; ItObject != _spawnedNodes.end(); )
							{
								this->definitelyDestroyNode(ItObject->second);
								ItObject = _spawnedNodes.erase(ItObject);
							}
						}
					}

					//update pass
					{
						this->updatePass(_currentLevel, simulationTime);
					}
				}

				if (_mainMenuLevel && _mainMenuLevel->_rootGroup)
					this->updatePass(_mainMenuLevel, simulationTime);

				if (_endMenuLevel && _endMenuLevel->_rootGroup)
					this->updatePass(_endMenuLevel, simulationTime);

				if (_hudLevel && _hudLevel->_rootGroup)
					this->updatePass(_hudLevel, simulationTime);

				if (_pauseMenuLevel && _pauseMenuLevel->_rootGroup)
					this->updatePass(_pauseMenuLevel, simulationTime);
			}
		}

		//Draw pass
		{
			this->drawPass(window, _currentLevel);
			if (_hudLevel && _hudLevel->_bShow && _hudLevel->_rootGroup)
			{
				this->drawPass(window, _hudLevel);
			}
			if (_mainMenuLevel && _mainMenuLevel->_bShow && _mainMenuLevel->_rootGroup)
			{
				this->drawPass(window, _mainMenuLevel);
			}
			if (_endMenuLevel && _endMenuLevel->_bShow && _endMenuLevel->_rootGroup)
			{
				this->drawPass(window, _endMenuLevel);
			}
			if (_pauseMenuLevel && _pauseMenuLevel->_bShow && _pauseMenuLevel->_rootGroup)
			{
				this->drawPass(window, _pauseMenuLevel);
			}
		}

		if (bShowDebug)
		{
			CProfiler::getInstance()->update(DeltaTime, &window);
		}

		if (bUpdate || _bPause)
		{
			simulationTime = 0.0f;
		}

		CScopeProfiler profiler("Display SFML");
		window.display();
	}
}

void CGame::startGame()
{
	_currentLevel->_bShow = true;
	_hudLevel->_bShow = true;
	_mainMenuLevel->_bShow = false;

	_gameTime = 0.0f;
	_gameStarted = true;
	_score = 0;
}

void CGame::validateMove( CNode* Node, Vector2f& Offset )
{
	if (_currentLevel && _currentLevel->_rootGroup)
	{
		CCollisionVisitor collVisitor(Node, Offset);
		collVisitor.visit(_currentLevel->_rootGroup);
		if (collVisitor._bCollide)
		{
			//Lose health
			Node->TakeDamage(collVisitor._colliderNode->getDamage());
			collVisitor._colliderNode->TakeDamage(Node->getDamage());
		}

		//manage out of screen issue
		FloatRect screenRect(0, 0, _screenSize.x, _screenSize.y);
		sf::Sprite* sprite = dynamic_cast<sf::Sprite*>(Node->getDrawable());
		if (sprite)
		{
			FloatRect SpriteRect = sprite->getGlobalBounds();
			Vector2f NewPos = sprite->getPosition();
			bool bTeleport = false;
			if (SpriteRect.left + SpriteRect.width < screenRect.left)
			{
				//Move to the right
				NewPos.x = _screenSize.x - 2;
				bTeleport = true;
			}
			else if (SpriteRect.left > screenRect.left + screenRect.width)
			{
				//Move to the right
				NewPos.x = 2;
				bTeleport = true;
			}
			if (SpriteRect.top + SpriteRect.height < screenRect.top)
			{
				//Move to the right
				NewPos.y = _screenSize.y - 2;
				bTeleport = true;
			}
			else if (SpriteRect.top > screenRect.top + screenRect.height)
			{
				//Move to the right
				NewPos.y = 2;
				bTeleport = true;
			}
			if (bTeleport)
			{
				if (Node->findFirstNodeBehaviorsOfClass<CMovingBehavior>())
				{
					//destroy it
					this->deleteNode(Node);
				}
				else
				{
					//teleport to the other size oh yeah and make a record of it
					CRecorder::getInstance()->makeRecordIfRegistered(Node);
					sprite->setPosition(NewPos);
					CRecorder::getInstance()->makeRecordIfRegistered(Node);
				}
				
			}
		}
	}
}

void CGame::notifyDeath( CNode* Node )
{
	if (Node->findFirstNodeBehaviorsOfClass<CPlayerControllerBH>())
	{
		// show end menu
		_endMenuLevel->_bShow = true;
	}
	else
	{
		//destroy it
		this->deleteNode(Node);
		this->addScore(Node->getScore());
	}
}

void CGame::updatePass( CLevel* level, float simulationTime )
{
	CScopeProfiler profiler("Update Visitor");
	CUpdateVisitor visitor(simulationTime);
	visitor.visit(level->_rootGroup);
}

void CGame::drawPass(sf::RenderTarget& RenderTarget, CLevel* level )
{
	CScopeProfiler profiler("Draw Pass");
	CDrawVisitor visitor;
	visitor.visit(level->_rootGroup);

	for (auto ItDrawable = visitor._sortedDrawables.begin(); ItDrawable != visitor._sortedDrawables.end(); ++ItDrawable)
	{
		RenderTarget.draw(**ItDrawable);
	}
}

void CGame::spawnNode( CNode* spawnedNode )
{
	//add to the scene
	if (_currentLevel && _currentLevel->_rootGroup)
	{
		_spawnedNodes.insert(std::make_pair(getGameTime(), spawnedNode));
		this->addToScene(spawnedNode);
	}
}

void CGame::definitelyDestroyNode( CNode* Object )
{
	CRecorder::getInstance()->removeRecordingObject(Object);
	CRecorder::getInstance()->removeReplayingObject(Object);
	//remove from scene
	auto itFound = std::find(_currentLevel->_rootGroup->_children.begin(), _currentLevel->_rootGroup->_children.end(), Object);
	if (itFound != _currentLevel->_rootGroup->_children.end())
	{
		_currentLevel->_rootGroup->_children.erase(itFound);
	}
	Object->clearNodeBehaviors();
	//delete node
	delete Object;
}

void CGame::deleteNode( CNode* deletedNode )
{
	//remove from scene
	auto itFound = std::find(_currentLevel->_rootGroup->_children.begin(), _currentLevel->_rootGroup->_children.end(), deletedNode);
	if (itFound != _currentLevel->_rootGroup->_children.end())
	{
		_currentLevel->_rootGroup->_children.erase(itFound);
		_deletedNodes.insert(std::make_pair(_gameTime, deletedNode));
	}
	deletedNode->_bDeleted = true;
}

void CGame::addToScene( CNode* node )
{
	_currentLevel->_rootGroup->_children.push_back(node);
	node->_bDeleted = false;
}

void CGame::togglePause()
{
	_pauseMenuLevel->_bShow = !_pauseMenuLevel->_bShow;
}

CDifficultyParams CGame::getActualDifficultyParams()
{
	if (_currentLevel)
	{
		for (auto ItDiff = _currentLevel->_difficultyParams.rbegin(); ItDiff != _currentLevel->_difficultyParams.rend(); ++ItDiff)
		{
			if (ItDiff->first <= _gameTime)
			{
				return ItDiff->second;
			}
		}
		return _currentLevel->_difficultyParams.rbegin()->second;
	}
	return CDifficultyParams();
}

void CGame::stopGame()
{
	_mainMenuLevel->_bShow = true;
	_gameStarted = false;
}

