#include "CNode.h"

#include "GameIncludes.h"

CNode::CNode(void) :
_drawable(nullptr),
_health(5.0f),
_bDeleted(false),
_score(0)
{
}


CNode::~CNode(void)
{
}

void CNode::update( float DeltaTime )
{
	for (auto It = _nodeBehaviors.begin(); It != _nodeBehaviors.end(); ++It)
	{
		(*It)->update(this, DeltaTime);
	}
}

void CNode::addNodeBehavior( CNodeBehavior* NodeBH )
{
	_nodeBehaviors.push_back(NodeBH);
}

void CNode::removeNodeBehavior( CNodeBehavior* NodeBH )
{
	auto itFound = std::find(_nodeBehaviors.begin(), _nodeBehaviors.end(), NodeBH);
	if (itFound != _nodeBehaviors.end())
	{
		_nodeBehaviors.erase(itFound);
	}
}

void CNode::forceMove( const Vector2f& Offset )
{
	Transformable* transformable = this->getTransformable();
	if (transformable)
	{
		transformable->move(Offset);
	}
}

void CNode::move( const Vector2f& Offset )
{
	Transformable* transformable = this->getTransformable();
	if (transformable)
	{
		Vector2f ActualMove = Offset;
		transformable->move(ActualMove);
		CGame::getInstance()->validateMove(this, ActualMove);
	}
}

void CNode::clearNodeBehaviors()
{
	for (auto ItNodeBH = _nodeBehaviors.begin(); ItNodeBH != _nodeBehaviors.end(); ++ItNodeBH)
	{
		delete *ItNodeBH;
	}
	_nodeBehaviors.clear();
}

void CNode::TakeDamage( float Damage )
{
	_health -= Damage;
	if (_health <= 0.0f)
	{
		CGame::getInstance()->notifyDeath(this);
	}
}

void CNode::rotate( float rotation )
{
	Transformable* transformable = this->getTransformable();
	if (transformable)
	{
		transformable->rotate(rotation);
	}
}

sf::Vector2f CNode::getXVector() const
{
	Vector2f xVector;
	const Transformable* transformable = this->getTransformable();
	if (transformable)
	{
		sf::Transform transform;
		transform.rotate(transformable->getRotation());
		xVector = transform.transformPoint(0.0f, 1.0f);
	}
	return xVector;
}

void CNode::setXVector( const Vector2f& direction )
{
	Transformable* transformable = this->getTransformable();
	if (transformable)
	{
		transformable->setRotation(CMath::GetAngleBetweenAxis(Vector2f(0.0, 1.0f), direction));
	}
}

void CNode::shoot( )
{
	Vector2f dir = getXVector();
	Vector2f pos = getTransformable()->getPosition() - (getTransformable()->getOrigin().y + 10.0f) * dir;

	//spawn projectile
	tinyxml2::XMLDocument XmlDoc;
	XmlDoc.LoadFile("media/templates/projectile.xml");
	XmlDoc.FirstChildElement()->SetAttribute("dirx", dir.x);
	XmlDoc.FirstChildElement()->SetAttribute("diry", dir.y);
	XmlDoc.FirstChildElement()->SetAttribute("x", pos.x);
	XmlDoc.FirstChildElement()->SetAttribute("y", pos.y);
	XmlDoc.FirstChildElement()->SetAttribute("speed", CGame::getInstance()->getActualDifficultyParams()._fireSpeed);
	CNode* node = CSerializer::getInstance()->loadFromXml(*XmlDoc.FirstChildElement());
	CGame::getInstance()->spawnNode(node);

	CRecorder::getInstance()->addRecordingObject(node);
	CRecorder::getInstance()->recordActionForObject(this, ACTION_SHOOT);
}

void CNode::doAction( EAction action )
{
	switch (action)
	{
	case ACTION_SHOOT:
		{
			this->shoot();
			break;
		}
	}
}