#pragma once

#include "stdafx.h"

class CNodeBehavior;

class CNode
{
public:
	CNode(void);
	virtual ~CNode(void);

	Drawable* getDrawable() { return _drawable; }
	void setDrawable(Drawable* drawable) { _drawable = drawable; }

	Transformable* getTransformable() { return dynamic_cast<Transformable*>(_drawable); }
	const Transformable* getTransformable() const { return dynamic_cast<const Transformable*>(_drawable); }

	virtual void update(float DeltaTime);

	void addNodeBehavior(CNodeBehavior* NodeBH);
	void removeNodeBehavior(CNodeBehavior* NodeBH);

	template<class T>
	T* findFirstNodeBehaviorsOfClass()
	{
		for (auto ItBH = _nodeBehaviors.begin(); ItBH != _nodeBehaviors.end(); ++ItBH)
		{
			if (dynamic_cast<T*>(*ItBH))
				return static_cast<T*>(*ItBH);
		}
		return nullptr;
	}

	void clearNodeBehaviors();

	//try to move it
	void move(const Vector2f& Offset);

	void setOriginalXml(const std::string& originalXml) { _originalXml = originalXml; }
	const std::string& getOriginalXml() const { return _originalXml; }
	void forceMove( const Vector2f& Offset );


	void TakeDamage( float Damage );
	float getHealth() const { return _health; }
	void setHealth(float health) { _health = health; }
	void rotate( float rotation );
	Vector2f getXVector() const;

	void shoot( );
	void doAction(EAction action);

	float getDamage() const { return _damage; }
	void setDamage(float damage) { _damage = damage; }
	void setXVector( const Vector2f& _direction );

	int getScore() const { return _score; }
	void setScore(int score) { _score = score; }

public:
	bool _bDeleted;
	Vector2f _velocity;

private:
	std::string _originalXml;

	Drawable* _drawable;

	std::vector<CNodeBehavior*> _nodeBehaviors;

	float _health;
	float _damage;
	int _score;
	
};

