#pragma once

#include "stdafx.h"

class CNode;

class CRecord
{
public:
	void saveFrom(CNode* object);
	void loadTo(CNode* object);

	static void loadFromInterpolationTo(CNode* object, const CRecord& Before, const CRecord& After, float Coeff);

public:
	Vector2f        _origin;                     ///< Origin of translation/rotation/scaling of the object
	Vector2f        _position;                   ///< Position of the object in the 2D world
	float           _rotation;                   ///< Orientation of the object, in degrees
	Vector2f        _scale;                      ///< Scale of the object

	float			_health;
	Vector2f _velocity;
};

class CRecorderHistory
{
public:
	void addRecord( float time, const CRecord& Record );

	void loadFromRecord(float time, CNode* Object);
	void removePassedRecords(float time);

	void addActionRecord(float time, EAction action);
	void replayActionRecordFor( CNode* first, float OldTime, float DeltaTime ) const;
	void removeActionRecordIfNeeded( CNode* Object, float OldTime, float NewTime );
private:
	std::multimap<float, EAction> _actionRecords;
	std::multimap<float, CRecord>  _records;
	std::pair<float, CRecord> _lastRecordSaved;
};

class CReplayingData
{
public:
	CNode* _basedOn;
	float _time;
};

class CRecorder
{
public:
	CRecorder();
	~CRecorder();

	void clearAll();

	void addRecordingObject(CNode* RecordingObject);

	void removeRecordingObject(CNode* RecordingObject);

	bool isRecordingObject(CNode* RecordingObject);

	void update(float DeltaTime);

	void recordObjects();

	void addReplayingObject(CNode* ReplayingObject, CNode* BasedOnRecordedObject, float Time);

	void removeReplayingObject(CNode* ReplayingObject);

	void replayObjects(float DeltaTime);

	CNode* findMoreRecentReplayingObjectsBasedOn(CNode* BasedOnNode);

public:
	static CRecorder* getInstance();
	bool isReplayingObject( CNode* Node );
	void unRecordObjects();
	void replayRecordedObject( CNode* RecordedObject );
	void recordActionForObject( CNode* Owner, EAction param2 );
	void manageActionRecords( float DeltaTime );
	void makeRecordIfRegistered( CNode* Node );

	bool isObjectBasedOnPlayedAround(CNode* BasedOnObject, float time, float marginTime);
public:
	std::vector<CNode*> _recordingObjects;

	std::map<CNode*, CReplayingData> _replayingObjects;

	std::map<CNode*, CRecorderHistory> _recordingHistory;

	float _recordInterval;

	float _recordRemainingTime;
};

